from osv import osv, fields
from tools.translate import _

class scrum_backlog(osv.osv):
    _inherit = 'project.scrum.product.backlog'

    _columns = {
        'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic account', ondelete='set null'),
    }

    def reparent_analytic_account(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        analytic_account = self.pool.get('account.analytic.account')
        for backlog in self.browse(cr, uid, ids, context):
            if backlog.sprint_id and backlog.sprint_id.analytic_account_id:
                parent_acc = None
                if backlog.sprint_id.analytic_account_id:
                    parent_acc = backlog.sprint_id.analytic_account_id.id
                elif backlog.project_id.analytic_account_id:
                    parent_acc = backlog.project_id.analytic_account_id.id

                if not backlog.analytic_account_id:
                    # Create an analytic account as child of the sprint analytic_account
                    new_account = analytic_account.copy(cr, uid, backlog.sprint_id.analytic_account_id.id, {'child_ids': []}, context)
                    analytic_account.write(cr, uid, new_account, {
                        'name': "[%d] %s" % (backlog.id, backlog.name),
                        'parent_id': parent_acc,
                    })
                    self.write(cr, uid, backlog.id, {'analytic_account_id': new_account})
                elif backlog.analytic_account_id.parent_id.id != backlog.sprint_id.analytic_account_id.id:
                    # Update the analytic_account parent to the analytic account in sprint
                    analytic_account.write(cr, uid, backlog.analytic_account_id.id, {
                        'parent_id': parent_acc,
                    })
            # Check all project tasks which need to rebook task work done
            task_ids = [ x.id for x in backlog.tasks_id ]
            self.pool.get('project.task').update_task_work(cr, uid, task_ids, context)
        return True

    def button_open(self, cr, uid, ids, context=None):
        self.reparent_analytic_account(cr, uid, ids, context)
        return super(scrum_backlog, self).button_open(cr, uid, ids, context)

    def write(self, cr, uid, ids, vals, context=None):
        res = super(scrum_backlog, self).write(cr, uid, ids, vals, context)
        if 'sprint_id' in vals:
            # Sprint (and thus parent analytic account) have changed !
            self.reparent_analytic_account(cr, uid, ids, context)
        return res

    #def unlink(self, cr, uid, ids, context=None):
    #   backlogs = self.browse(cr, uid, ids, context)
    #   analytic_ids = [ x.analytic_account_id.id for x in backlogs ]
    #   res = super(scrum_backlog, self).unlink(cr, uid, ids, context)
    #   self.pool.get('account.analytic.account').unlink(cr, uid, analytic_ids, context)
    #   return res
scrum_backlog()

class scrum_sprint(osv.osv):
    _inherit = 'project.scrum.sprint'

    _columns = {
        'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic account', ondelete='set null'),
    }

    def reparent_analytic_account(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]

        analytic_account = self.pool.get('account.analytic.account')
        for sprint in self.browse(cr, uid, ids, context):
            parent_acc = sprint.project_id.analytic_account_id.id

            if not sprint.analytic_account_id:
                # Create an analytic account
                new_account = analytic_account.copy(cr, uid, sprint.project_id.analytic_account_id.id, {'child_ids': []}, context)
                analytic_account.write(cr, uid, new_account, {
                    'name': sprint.name,
                    'parent_id': parent_acc,
                })
                self.write(cr, uid, sprint.id, {'analytic_account_id': new_account})
            elif sprint.analytic_account_id.parent_id.id != sprint.project_id.analytic_account_id.id:
                # Update analytic account parent to the project analytic account
                analytic_account.write(cr, uid, sprint.analytic_account_id.id, {
                    'parent_id': parent_acc,
                })

            # Check all underlying backlogs
            self.pool.get('project.scrum.product.backlog').reparent_analytic_account(cr, uid, [ x.id for x in sprint.backlog_ids ], context)
        return True

    def button_open(self, cr, uid, ids, context=None):
        self.reparent_analytic_account(cr, uid, ids, context)
        return super(scrum_sprint, self).button_open(cr, uid, ids, context)

    def write(self, cr, uid, ids, vals, context=None):
        res = super(scrum_sprint, self).write(cr, uid, ids, vals, context)
        if 'project_id' in vals:
            # Project (and thus the parent analytic account) have been changed !
            self.reparent_analytic_account(cr, uid, ids, context)
        return res
scrum_sprint()

class project_task(osv.osv):
    _inherit = 'project.task'

    def update_task_work(self, cr, uid, ids, context):
        for task in self.browse(cr, uid, ids, context):
            work_ids = [ x.id for x in task.work_ids ]
            self.pool.get('project.task.work').set_scrum_analytic(cr, uid, work_ids, context)
        return True

    def write(self, cr, uid, ids, vals, context=None):
        res = super(project_task, self).write(cr, uid, ids, vals, context)
        if 'product_backlog_id' in vals:
            # User has changed the backlog, update the task work entries if present
            self.update_task_work(cr, uid, ids, context)
        return res
project_task()

class project_task_work(osv.osv):
    _inherit = 'project.task.work'

    def set_scrum_analytic(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]

        for task_work in self.browse(cr, uid, ids, context):
            if task_work.hr_analytic_timesheet_id:
                # We change the analytic timesheet account to match the backlog analytic account (or project account)
                aal = self.pool.get('account.analytic.line')
                acc_id = None
                if task_work.task_id.product_backlog_id:
                    acc_id = task_work.task_id.product_backlog_id.analytic_account_id.id
                else:
                    acc_id = task_work.task_id.project_id.analytic_account_id.id
                aal.write(cr, uid, [task_work.hr_analytic_timesheet_id.line_id.id], {
                    'account_id': acc_id,
                })
        return True

    def create(self, cr, uid, vals, context=None):
        res = super(project_task_work, self).create(cr, uid, vals, context)
        self.set_scrum_analytic(cr, uid, res, context)
        return res

    def write(self, cr, uid, ids, vals, context=None):
        res = super(project_task_work, self).write(cr, uid, ids, vals, context)
        self.set_scrum_analytic(cr, uid, ids, context)
        return res
project_task_work()
